/* $Id: notice.c,v 1.2 1998/11/03 04:49:08 bill Exp $ */
/* Notice Dialog Sample 
 *
 * By Bill Baxter
 * For the Avigo Freeware SDK v0.91b2
 */

#include <avsys.h>
#include "notice.h"

#define GLOMXY(x,y)          ((x)<<8)|((y)&0xff)
#define setDrawArea(x,y,a,b) SetDrawArea(GLOMXY(x,y),GLOMXY(a,b))
#define BUTTON_BOTTOM_OFFSET 6
#define BUTTON_HEIGHT 16

/* This tells us the height of a font from its number */
static const int FontHeightAssoc[] = { 11, 11, 14, 14, 7 };

/* Box corner bitmaps */
static const unsigned char corners[] = { 
    0x34, 0x88, /* UL */
    0xC0, 0x61, /* UR */
    0x8A, 0x21, /* LL */
    0x53, 0x68  /* LR */
};

/* The backup buffer is located at this fixed address in the Avigo! */
#define bNoticeBACKUP_BUF ((char*)0xa7d0)

#define MSGBUF_SIZE 100
static char msgbuf[MSGBUF_SIZE];



void bNoticeStartup()
{
    /* No static data to initialize currently.
     * In fact, as long as there is no malloc()
     * there probably never will be.  
     */ 
}

void bNoticeInit(bNotice *notice)
{
    /* Set all default values for the notice */
    notice->x1 = 20;
    notice->x2 = 140;
    notice->y1 = 50;
    notice->y2 = 120;
    notice->font = PRPFONT11N;
    notice->message = "";
    notice->bmessage = "OK";
    notice->doBackup = 1;
}

void bNoticeButtonLoop(bNotice *notice)
{
    int key;
    unsigned char penx, peny;
    int inside_button = 0;
    int last = 0;

    while (1) {
        key = GetKey(&penx, &peny);
        if (key != PENDOWN && last == PENDOWN && inside_button) {
            /* penup with button pressed! */
            PenBeep();
            break;
        }
        if (key == PENDOWN)
        {
            if (penx >= notice->bx1 && penx <= notice->bx2 &&
                peny >= notice->by1 && peny <= notice->by2)
            {
                if (!inside_button)
                    bNoticeButtonDraw(notice, 1);
                inside_button = 1;
                last = key;
                continue;
            } else if (inside_button) {
                bNoticeButtonDraw(notice, 0);
            }
        }
        inside_button = 0;
        last = key;
    }

    /* Clear queued input */
    while (GetKey(&penx, &peny)) {
        key = 0;
    }
}


void bNoticeOpen(bNotice *notice)
{
    if (notice->doBackup)
    {
        BackupWindow(notice->x1, notice->y1, notice->x2, notice->y2,
                     bNoticeBACKUP_BUF);
    }    

    bNoticeDraw(notice);

    bNoticeButtonLoop(notice);

    if (notice->doBackup)
    {
        RestoreWindow(notice->x1, notice->y1, bNoticeBACKUP_BUF, 0);
    }    
}

void bNoticeClose(bNotice *notice)
{
    /* Not implemented yet */
}

bNoticeButtonDraw(bNotice *notice, int hilite)
{
    int x, y, a, b;
    int vspace;
    int d, w;
    int savesize = GetLineSize();
    int savefont = GetFontType();
    int font = PRPFONT11B;  /* Could become notice variable */
    int rad = BUTTON_HEIGHT>>1;

    SetFontType(font);

    if (savesize != 0)
        SetLineSize(0);
    
    w = GetStringLength(notice->bmessage, font) + BUTTON_HEIGHT;
    x = notice->x1;
    y = notice->y2 - BUTTON_BOTTOM_OFFSET - BUTTON_HEIGHT;
    a = notice->x2;
    b = notice->y2 - BUTTON_BOTTOM_OFFSET;

    d = ((a - x) - w)>>1;
    x += d;
    a -= d;
    
    /* Cache for hit test later */
    notice->bx1 = x;
    notice->bx2 = a;
    notice->by1 = y;
    notice->by2 = b;

    if (hilite) {
        FillCircle(x+rad,y+rad, rad, DRAW_BLACK);
        FillCircle(a-rad,b-rad, rad, DRAW_BLACK);
    } else {
        setDrawArea(x,y,a,b);
        FillRect(x,y,a,b,DRAW_WHITE);
        setDrawArea(x,y,x+rad,b);
        DrawCircle(x+rad,y+rad, rad, DRAW_BLACK);
        setDrawArea(a-rad,y,a,b);
        DrawCircle(a-rad,b-rad, rad, DRAW_BLACK);
        setDrawArea(0, 0, LCD_WIDTH-1, LCD_HEIGHT-1);
    }
    DrawLine(x+rad, y, a-rad, y, DRAW_BLACK);
    DrawLine(x+rad, b, a-rad, b, DRAW_BLACK);

    vspace = BUTTON_HEIGHT - FontHeightAssoc[font] + 1;
    if (hilite) {
        FillRect(x+rad+1, y+1, a-rad-1, b-1, DRAW_BLACK);
        WriteString(x+rad, y+(vspace>>1), notice->bmessage, DRAW_WHITE);
    } else {
        WriteString(x+rad, y+(vspace>>1), notice->bmessage, DRAW_BLACK);
    }

    if (savesize != 0)
        SetLineSize(savesize);
    if (savefont != font) 
        SetFontType(savefont);
}

void bNoticeMessageDraw(bNotice *notice)
{
    char *msg = notice->message;
    char *p, *drawme;
    char savechar;
    int fh;
    int h=0;
    int w;
    int boxw;
    int savefont = GetFontType();

    boxw = notice->x2 - notice->x1;

    if (!msg)
        msg = "Null message";

    SetFontType(notice->font);
    fh = FontHeightAssoc[notice->font];
    
    /* Write on the virtual screen, keeping track of
     * how high the text is, then copy it over to the 
     * real world, centered vertically.
     */

    SetToVirtualLCD(0);
    ClearLCD(0);
    while (msg && *msg) {
        if ( (p = strstr(msg, "\n")) != (char*)0) {
            strncpy(msgbuf, msg, p-msg);
            *(msgbuf+(p-msg)) = '\0';
            drawme = msgbuf;
        }
        else {
            drawme = msg;
        }
        w = GetStringLength(drawme, notice->font);
        WriteString((boxw-w)>>1, h, drawme, DRAW_BLACK);
        if (p) {
            msg = p+1;
        }
        else {
            msg = (char*)0;
        }
        h += fh;
    }
    SetToRealLCD(0);

    /* reuse fh for the amount of vertical margin */
    fh = (notice->y2-notice->y1-BUTTON_HEIGHT-BUTTON_BOTTOM_OFFSET-h)>>1;
    SwapLCD(GLOMXY(notice->x1,notice->y1+fh), 0x00, 
            GLOMXY(boxw,h), 0,REALTOVIRT);

    SetFontType(savefont);
}


/* miniblit
 *   A really simple 4x4 blit (used for drawing corners of box)
 */
static void miniblit(int x, int y, unsigned char* bits)
{
    int i,j;
    unsigned char mask = 0x80;
    for (j=0; j<4; j++) {
        if (j==2) {
            bits++;
            mask = 0x80;
        }
        for (i=0; i<4; i++) {
            if (*bits & mask)
                DrawDot(x+i, y+j, DRAW_BLACK);
            mask >>= 1;
        }
    }
}


void bNoticeDraw(bNotice *notice)
{
    int x, y, a, b, w;
    int savesize = GetLineSize();

/*    int rad = 8; */
    x = notice->x1;
    y = notice->y1;
    a = notice->x2;
    b = notice->y2;
    
    SetLineSize(0);
    FillRect(x, y, a, b, DRAW_WHITE);

    bNoticeButtonDraw(notice, 0);
    bNoticeMessageDraw(notice);

    /* Draw corners */
    miniblit(x+1, y+1, corners);    /* UL */
    miniblit(a-4, y+1, corners+2);  /* UR */
    miniblit(x+1, b-4, corners+4);  /* LL */
    miniblit(a-4, b-3, corners+6);  /* LR */

    /* Draw sides */
    DrawLine(x+5, y, a-5, y, DRAW_BLACK); /* top */
    DrawLine(x+5, b, a-5, b, DRAW_BLACK); /* bottom */
    DrawLine(x, y+5, x, b-5, DRAW_BLACK); /* left */
    DrawLine(a, y+5, a, b-5, DRAW_BLACK); /* right */
    SetLineSize(1);
    DrawLine(x+5, b-2, a-5, b-2, DRAW_GRAY); /* bottom shadow */
    DrawLine(a-2, y+5, a-2, b-5, DRAW_GRAY); /* right shadow */

#if 0
    /* An attempt to draw corners with circles -- never looked right */
    SetLineSize(0);
    setDrawArea(x, y, x+rad, y+rad);         /* Upper left */
    DrawCircle(x+rad, y+rad, rad, DRAW_BLACK);
    setDrawArea(a-rad, y, a, y+rad);         
    DrawCircle(a-rad, y+rad, rad, DRAW_BLACK);
    setDrawArea(x, b-rad, a+rad, b);
    DrawCircle(x+rad, b-rad, rad, DRAW_BLACK);
    setDrawArea(a-rad, b-rad, a, b);         /* Lower right */
    DrawCircle(a-rad, b-rad, rad, DRAW_BLACK);
#endif

    setDrawArea(0, 0, LCD_WIDTH-1, LCD_HIGH-1);

    SetLineSize(savesize);
}







